﻿using HarmonyLib;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using UnityEngine;
using UnityEngine.UI;
using HMLLibrary;
using UnityEngine.SceneManagement;
using System.IO;
using Steamworks;
using System.Runtime.CompilerServices;
using System.Text.RegularExpressions;
using System;
using _Raft;
using RaftModLoader;
using FMOD;
using Debug = UnityEngine.Debug;

public class the_tweaker_mod : Mod
{
    private Harmony harmonyInstance;
    private const string HARMONY_ID = "the_tweaker_mod";

    public void Start()
    {
        ComponentManager<the_tweaker_mod>.Value = this;
        harmonyInstance = new Harmony(HARMONY_ID);
        harmonyInstance.PatchAll(Assembly.GetExecutingAssembly());
        this.BuildRecipes(); Debug.Log("'The Tweaker Mod' recipes have been successfully constructed!");
        this.ChangeVanillaMembers(); Debug.Log("'The Tweaker Mod' has changed some vanilla members successfully!");
        Debug.Log("'The Tweaker Mod' is now activated!");
    }

    public void Update()
    {
        this.BuildRecipes();
        this.ChangeVanillaMembers();
    }

    //Mod Unloading
    public void OnModUnload()
    {
        harmonyInstance.UnpatchAll(HARMONY_ID);
        Debug.Log("'The Tweaker Mod' is now deactivated!");
    }

    //World Loaded Event
    public override void WorldEvent_WorldLoaded()
    {
        this.BuildRecipes(); Debug.Log("'The Tweaker Mod' recipes have been successfully constructed!");
        this.ChangeVanillaMembers(); Debug.Log("'The Tweaker Mod' has changed some vanilla members successfully!");
        Debug.Log("The world has been loaded!");
    }

    //Recipe Manager
    public void BuildRecipes()
    {
        CreateRecipe(ItemManager.GetItemByName("DevSpear"), 1, CraftingCategory.Weapons, allowItems(new string[] { "TitaniumIngot", "CopperIngot", "Hinge", "Bolt", "Rope" }, new int[] { 30, 50, 5, 10, 10 }));
        CreateRecipe(ItemManager.GetItemByName("ExplosiveGoo"), 2, CraftingCategory.Resources, allowItems(new string[] { "VineGoo", "Flower_White", "Flower_Blue", "Flower_Yellow", "BioFuel" }, new int[] { 5, 5, 5, 5, 5 }));
        CreateRecipe(ItemManager.GetItemByName("MetalDetector"), 1, CraftingCategory.Tools, allowItems(new string[] { "CircuitBoard", "MetalIngot", "Bolt", "Nail" }, new int[] { 3, 5, 2, 4 }));
        CreateRecipe(ItemManager.GetItemByName("Seed_Palm"), 1, CraftingCategory.FoodWater, allowItems(new string[] { "Coconut" }, new int[] { 1 }));
        CreateRecipe(ItemManager.GetItemByName("Seed_Mango"), 1, CraftingCategory.FoodWater, allowItems(new string[] { "Mango" }, new int[] { 1 }));
        CreateRecipe(ItemManager.GetItemByName("Seed_Banana"), 1, CraftingCategory.FoodWater, allowItems(new string[] { "Banana" }, new int[] { 1 }));
        CreateRecipe(ItemManager.GetItemByName("Seed_Watermelon"), 4, CraftingCategory.FoodWater, allowItems(new string[] { "Watermelon" }, new int[] { 1 }));
        CreateRecipe(ItemManager.GetItemByName("Seed_Pineapple"), 2, CraftingCategory.FoodWater, allowItems(new string[] { "Pineapple" }, new int[] { 1 }));
        CreateRecipe(ItemManager.GetItemByName("Seed_Flower_Red"), 1, CraftingCategory.FoodWater, allowItems(new string[] { "Flower_Red" }, new int[] { 1 }));
        CreateRecipe(ItemManager.GetItemByName("Seed_Flower_White"), 1, CraftingCategory.FoodWater, allowItems(new string[] { "Flower_White" }, new int[] { 1 }));
        CreateRecipe(ItemManager.GetItemByName("Seed_Flower_Black"), 1, CraftingCategory.FoodWater, allowItems(new string[] { "Flower_Black" }, new int[] { 1 }));
        CreateRecipe(ItemManager.GetItemByName("Seed_Flower_Blue"), 1, CraftingCategory.FoodWater, allowItems(new string[] { "Flower_Blue" }, new int[] { 1 }));
        CreateRecipe(ItemManager.GetItemByName("Seed_Flower_Yellow"), 1, CraftingCategory.FoodWater, allowItems(new string[] { "Flower_Yellow" }, new int[] { 1 }));
        CreateRecipe(ItemManager.GetItemByName("Seed_Strawberry"), 2, CraftingCategory.FoodWater, allowItems(new string[] { "Strawberry" }, new int[] { 1 }));
        CreateRecipe(ItemManager.GetItemByName("Jar_Honey"), 1, CraftingCategory.FoodWater, allowItems(new string[] { "HoneyComb", "Glass" }, new int[] { 3, 1 }));
    }

    private CostMultiple[] allowItems(string[] validItemNames, int[] itemCounts)
    {
        var length = (int)Math.Min(validItemNames.Length, itemCounts.Length);
        var costs = new List<CostMultiple>();
        for (int i = 0; i < length; i++)
            if (ItemManager.GetItemByName(validItemNames[i]))
                costs.Add(new CostMultiple(new Item_Base[] { ItemManager.GetItemByName(validItemNames[i]) }, itemCounts[i])); 
        return costs.ToArray();
    }

    public static void CreateRecipe(Item_Base pResultItem, int pAmount, object pCategory, CostMultiple[] cost)
    {
        Traverse.Create(pResultItem.settings_recipe).Field("craftingCategory").SetValue(pCategory);
        Traverse.Create(pResultItem.settings_recipe).Field("amountToCraft").SetValue(pAmount);
        Traverse.Create(pResultItem.settings_recipe).Field("learnedFromBeginning").SetValue(true);
        Traverse.Create(pResultItem.settings_recipe).Field("learned").SetValue(true);
        pResultItem.settings_recipe.NewCost = cost;
    }

    //Vanilla Member Manager
    public void ChangeVanillaMembers()
    {
        if (RAPI.GetLocalPlayer() != null)
        {
            Sprinkler sprinkler = FindObjectOfType<Sprinkler>(); Traverse.Create(sprinkler).Field("batteryUsesPerPlot").SetValue(0);
            Electric_Purifier electric_Purifier = FindObjectOfType<Electric_Purifier>(); Traverse.Create(electric_Purifier).Field("batteryDrainPerPump").SetValue(0);
        }
    }

    //Glass and Bowl Consumption Return
    private static bool IsBowlPlateItem(Item_Base item)
    {
        return item.name.Contains("Claybowl") || item.name.Contains("ClayPlate");
    }
    private static bool IsJarItem(Item_Base item)
    {
        return item.name.Contains("Jar");
    }

    public void OnConsumeItem(Item_Base item)
    {
        if (IsJarItem(item))
        {
            RAPI.GiveItem(ItemManager.GetItemByName("Glass"), 1);
        }
        if (IsBowlPlateItem(item))
        {
            RAPI.GiveItem(ItemManager.GetItemByName("Claybowl_Empty"), 1);
        }
    }
    [HarmonyPatch(typeof(PlayerStats)), HarmonyPatch("Consume")]
    internal class ConsumePatch
    {
        private static void Postfix(PlayerStats __instance, Item_Base edibleItem)
        {
            if (__instance.GetComponent(typeof(Network_Player)) == RAPI.GetLocalPlayer())
            {
                ComponentManager<the_tweaker_mod>.Value.OnConsumeItem(edibleItem);
            }
        }
    }

    //Box Loot
    public override void LocalPlayerEvent_PickupItem(PickupItem item)
    {
        if (item.name.Contains("Barrel") || item.name.Contains("Crate") || item.name.Contains("Safe") || item.name.Contains("Case"))
        {
            BoxLoot();
        }
    }
    public void BoxLoot()
    {
        var rand = new System.Random();
        var slots = rand.Next(1, 6);
        switch (slots)
        {
            case 1: RAPI.GiveItem(ItemManager.GetItemByName("Nail"), 3); RAPI.GiveItem(ItemManager.GetItemByName("Hinge"), 2); RAPI.GiveItem(ItemManager.GetItemByName("Bolt"), 4); break;
            case 2: RAPI.GiveItem(ItemManager.GetItemByName("TitaniumOre"), 2); RAPI.GiveItem(ItemManager.GetItemByName("MetalOre"), 4); RAPI.GiveItem(ItemManager.GetItemByName("CopperOre"), 3); break;
            case 3: RAPI.GiveItem(ItemManager.GetItemByName("CopperIngot"), 3); RAPI.GiveItem(ItemManager.GetItemByName("Glass"), 4); RAPI.GiveItem(ItemManager.GetItemByName("Hinge"), 2); RAPI.GiveItem(ItemManager.GetItemByName("Bolt"), 2); break;
            case 4: RAPI.GiveItem(ItemManager.GetItemByName("TitaniumIngot"), 3); QuestItemManager.AddQuestItem(QuestItemType.Tangaroa_Token, 4, true); break;
            case 5: RAPI.GiveItem(ItemManager.GetItemByName("BioFuel"), 3); QuestItemManager.AddQuestItem(QuestItemType.Balboa_WildBerries, 5, true); break;
            case 6: RAPI.GiveItem(ItemManager.GetItemByName("TitaniumIngot"), 1); RAPI.GiveItem(ItemManager.GetItemByName("CopperIngot"), 3); RAPI.GiveItem(ItemManager.GetItemByName("MetalIngot"), 2); break;
            default:this.BoxLoot();break;
        }
    }

    //Shark Behavior

    [HarmonyPatch(typeof(AI_State_Attack_Block_Shark), "FindBlockToAttack")]
    public static class AIStateAttackBlockShark_FindBlockToAttack_Patch { private static void Postfix(ref Block __result) { __result = null; } }
}